function create_DB(root)
    if nargin < 1 || isempty(root)
        error('Please indicate path to the database.');
    end
    
    % Create directory
    [s,m] = mkdir(root);
    d = fileparts(mfilename('fullpath'));
    copyfile(fullfile(d, 'DB'), root);    
    addpath(fullfile(root, 'utils'));
    ssem_compile();

    params = ssem_set_params_sub(root);  
        
    % Download videos
    fail = download_vids(d, params);
    
    % List videos
    list_vids(params);

    % Extract frames, background and background image
    extract_backs(params);            

    % Split DB into pieces with same label repartition
    make_splits(params);  

    if ~isempty(fail)
        fprintf('\n********\n\nFollowing links could not be downloaded:\n%s\n********\n', sprintf('%s\n', fail{:}));  
        fprintf('Please contact the authors and attach the file %s.\n', fullfile(params.root, 'missing_URLs.txt'));
    end
    
    vids = ssem_load_subvideos(params);
    fprintf('\n%d videos extracted.\n', length(vids));
    if length(vids) ~= 146
        warning('Original database used in paper "Scene semantics from long-term observation of people" was based on 146 videos. Results might be similar but not identical to those reported in the paper.');
        fprintf('\n');
    else
        fprintf('Database created successfully !\n');
    end    
end

function urls = download_vids(d, params)
    fid = fopen(fullfile(params.root, 'URLs.txt'), 'r');
    urls = textscan(fid, '%s'); 
    fclose(fid);
    urls = urls{1};
    
    vidsDir = fullfile(params.root, 'vids');
    currDir = cd();
    [s,m] = mkdir(vidsDir);
    
    cd(vidsDir);
        
    delete('*.part');
    fail = [];
        
    prefixYoutube = 'http://www.youtube.com/watch?v=';
    for i = 1 : length(urls)        
        if strcmp(prefixYoutube, urls{i}(1 : min(length(urls{i}), length(prefixYoutube))))
            vid = urls{i}((length(prefixYoutube) + 1) : end);
            if ~isempty(dir([vid '.*']))
                fprintf('Skipping %s\n', vid);
                continue;
            else
                fprintf('Downloading %s\n', vid);
                while 1
                    [s r] = system(sprintf('python %s -q %s', fullfile(d, '3rdparty', 'youtube-dl', 'youtube-dl.py'), urls{i}));                                          
                    if ~isempty(strfind(r, 'ERROR')) || ~isempty(strfind(r, 'python'))
                        fprintf(r);
                    end
                    if s ~= 0 && ~isempty(strfind(r, '402'))
                        fprintf('Blocked by Youtube, retrying within 1 hour.\n');
                        pause(3600);
                    else                        
                        break;
                    end
                end
            end
        end
        if s ~= 0
            fail = [fail i];               
        else
            pause(30); % trying to avoid to get blocked by youtube.
        end
    end
        
    cd(currDir);
    
    % Write missing links
    urls = urls(fail);
    fid = fopen(fullfile(params.root, 'missing_URLs.txt'), 'w');    
    fprintf(fid, '%s\n', urls{:});
    fclose(fid);    
    
    if isempty(fail)
        fprintf('\n********\nDownload successful !\n');
    else
        fprintf('\n********\n\nFollowing links could not be downloaded:\n%s\n********\n', sprintf('%s\n', urls{fail}));        
    end
end

function list_vids(params)
    fid = fopen(fullfile(params.root, params.video_file), 'w');
    vidsDir = fullfile(params.root, 'vids');
    vids = dir(vidsDir);
    for i = 1 : length(vids)
        if ~vids(i).isdir
            fprintf(fid, '%s\n', cleanname(vids(i).name));                
        end
    end
    fclose(fid);
end

function vid = cleanname(vid)
    [aaa, vid] = fileparts(vid);
    vid = strrep(vid, '-', '_');
end
