#include <math.h>
#include <mex.h>

#define DEBUG 0
#define DEBUGPIX (int)(0.25*w*h + 0.66*h)

#define MULT_VAR 10
#define MERGE_DENSITY 0.1
#define MIN_SIGMA 90
#define NEWVAR 8000
#define NEWWEIGHT 0.1
#define VARTH  (2.5 * 2.5)

#define PIX(p,i,j,h)            p[(i) * (h) + (j)]
#define PIXC_openN(p,i,j,w,h)   ((i == 0   || j == h-1 || PIX(p,i-1,j,h) * PIX(p,i,j+1,h) >= 0) &&\
                                 (i == w-1 || j == h-1 || PIX(p,i,j+1,h) * PIX(p,i+1,j,h) >= 0) && (\
                                     (j < h-1 && PIX(p,i,j,h) * PIX(p,i,j+1,h) <= 0) ||\
                                     (i < w-1 && PIX(p,i,j,h) * PIX(p,i+1,j,h) <= 0) ||\
                                     (i > 0   && PIX(p,i,j,h) * PIX(p,i-1,j,h) <= 0)))
#define PIXC_openS(p,i,j,w,h)   ((i == 0   || j == 0 || PIX(p,i-1,j,h) * PIX(p,i,j-1,h) >= 0) &&\
                                 (i == w-1 || j == 0 || PIX(p,i,j-1,h) * PIX(p,i+1,j,h) >= 0) && (\
                                     (j > 0   && PIX(p,i,j,h) * PIX(p,i,j-1,h) <= 0) ||\
                                     (i < w-1 && PIX(p,i,j,h) * PIX(p,i+1,j,h) <= 0) ||\
                                     (i > 0   && PIX(p,i,j,h) * PIX(p,i-1,j,h) <= 0)))
#define PIXC_openE(p,i,j,w,h)   ((i == 0 || j == h-1 || PIX(p,i,j+1,h) * PIX(p,i-1,j,h) >= 0) &&\
                                 (i == 0 || j == 0   || PIX(p,i-1,j,h) * PIX(p,i,j-1,h) >= 0) && (\
                                     (i > 0   && PIX(p,i,j,h) * PIX(p,i-1,j,h) <= 0) ||\
                                     (j > 0   && PIX(p,i,j,h) * PIX(p,i,j-1,h) <= 0) ||\
                                     (j < h-1 && PIX(p,i,j,h) * PIX(p,i,j+1,h) <= 0)))                                                                          
#define PIXC_openW(p,i,j,w,h)   ((i == w-1 || j == h-1 || PIX(p,i,j+1,h) * PIX(p,i+1,j,h) >= 0) &&\
                                 (i == w-1 || j == 0   || PIX(p,i+1,j,h) * PIX(p,i,j-1,h) >= 0) && (\
                                     (i < w-1 && PIX(p,i,j,h) * PIX(p,i+1,j,h) <= 0) ||\
                                     (j > 0   && PIX(p,i,j,h) * PIX(p,i,j-1,h) <= 0) ||\
                                     (j < h-1 && PIX(p,i,j,h) * PIX(p,i,j+1,h) <= 0)))
#define PIXC_openN_sum(p,i,j,h) (PIX(p,i-1,j,h) + PIX(p,i,j+1,h) + PIX(p,i+1,j,h))
#define PIXC_openS_sum(p,i,j,h) (PIX(p,i-1,j,h) + PIX(p,i,j-1,h) + PIX(p,i+1,j,h))
#define PIXC_openE_sum(p,i,j,h) (PIX(p,i,j+1,h) + PIX(p,i-1,j,h) + PIX(p,i,j-1,h))
#define PIXC_openW_sum(p,i,j,h) (PIX(p,i,j+1,h) + PIX(p,i+1,j,h) + PIX(p,i,j-1,h))


double min(double a, double b)
{
  return (a < b) ? a : b;
}

double max(double a, double b)
{
  return (a > b) ? a : b;
}

double get_var(const double *sigma, int offset)
{
	const double vr = sigma[0];
	const double vg = sigma[offset];
  const double vb = sigma[offset * 2];
  return vr * vg * vb;
}

double get_dist2(const double *x, const double *y, int offset)
{
	const double dr = x[0]          - y[0];
	const double dg = x[offset]     - y[offset];
  const double db = x[offset * 2] - y[offset * 2];
  return dr * dr + dg * dg + db * db;
}

double get_density(const double *x, const double *sigma, const double *mu, int offset)
{
	const double dr = x[0]          - mu[0];
	const double dg = x[offset]     - mu[offset];
  const double db = x[offset * 2] - mu[offset * 2];
	const double vr = sigma[0];
	const double vg = sigma[offset];
  const double vb = sigma[offset * 2];  
	return exp(-(dr * dr / vr + dg * dg / vg + db * db / vb) / 2.); 
}

double get_normalized_density(const double *x, const double *sigma, const double *mu, int offset)
{
	const double dr = x[0]          - mu[0];
	const double dg = x[offset]     - mu[offset];
  const double db = x[offset * 2] - mu[offset * 2];
	const double vr = sigma[0];
	const double vg = sigma[offset];
  const double vb = sigma[offset * 2];
  const double det = vr * vg * vb;
	return exp(-(dr * dr / vr + dg * dg / vg + db * db / vb) / 2.) / sqrt(pow(2. * M_PI, 3) * det); 
}

typedef struct gparams
{
	double *weights;
	double *stddev;
	int ID;
	int offset;
} gparams;

int params_compare(const gparams *a, const gparams *b)
{
  const double *stddev = a->stddev;
  const double *weights = a->weights;
  const int offset = a->offset;
  
	double ra = weights[offset * a->ID] / stddev[a->ID];
	double rb = weights[offset * b->ID] / stddev[b->ID];	
	return ra > rb ? -1 : (ra == rb ? 0 : 1);
}

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
  double *img;
  double alpha;
  double T;
  double *sigma;
  double *mu;
  double *weights;
  
  mwSize *dimsimg;
  mwSize *dimssigma;
  mwSize *dimsmu;
  mwSize *dimsweights;
  
  double *outsigma;
  double *outmu;
  double *outweights;
  double *foreground;
  double *background;
    
  int h, w, K;
  int offset, offset3;
    
  int *match;
  double *stddev;
  gparams *params;
  int i, j, k, l;
  double sumback;
  double sumweight;
  double nmatch;
  double ratio;
  
    
  /* check proper input and output */
  if(nrhs!=6)
    mexErrMsgTxt("Input arguments required: image, alpha, T, sigma, mu, weights");
  else if(nlhs != 5 && nlhs != 4)
    mexErrMsgTxt("4 or 5 output arguments required: sigma, mu, weights, foreground, background");
      
  if(mxGetClassID(prhs[0]) != mxDOUBLE_CLASS || mxGetClassID(prhs[1]) != mxDOUBLE_CLASS || mxGetClassID(prhs[2]) != mxDOUBLE_CLASS ||
     mxGetClassID(prhs[3]) != mxDOUBLE_CLASS || mxGetClassID(prhs[4]) != mxDOUBLE_CLASS || mxGetClassID(prhs[5]) != mxDOUBLE_CLASS)
    mexErrMsgTxt("Input should be double.");
  if(mxGetNumberOfDimensions(prhs[0]) != 3)
    mexErrMsgTxt("images: wrong dimension number");  	
  if(mxGetNumberOfElements(prhs[1]) != 1)  
  	mexErrMsgTxt("alpha: should be scalar");
  if(mxGetNumberOfElements(prhs[2]) != 1)  
  	mexErrMsgTxt("T: should be scalar");  	  	  	
  if (mxGetNumberOfDimensions(prhs[3]) != 4)
	  mexErrMsgTxt("sigma: wrong dimension number");  	
 	if (mxGetNumberOfDimensions(prhs[4]) != 4)
 	  mexErrMsgTxt("mu: wrong dimension number");  	
  if (mxGetNumberOfDimensions(prhs[5]) != 3)
	  mexErrMsgTxt("weights: wrong dimension number");  	
                
  img = mxGetPr(prhs[0]);  
  alpha = mxGetScalar(prhs[1]);  
  T = mxGetScalar(prhs[2]);  
  sigma = mxGetPr(prhs[3]);
  mu = mxGetPr(prhs[4]);  
  weights = mxGetPr(prhs[5]);  
  
  dimsimg = mxGetDimensions(prhs[0]);
  dimssigma = mxGetDimensions(prhs[3]);
  dimsmu = mxGetDimensions(prhs[4]);
  dimsweights = mxGetDimensions(prhs[5]);
    
  if (dimsimg[2] != 3)
	  mexErrMsgTxt("image: wrong dimensions."); 
	if (dimsimg[0] != dimssigma[0] || dimsimg[1] != dimssigma[1] || dimssigma[2] != 3)
	  mexErrMsgTxt("sigma: wrong dimensions."); 
	if (dimsimg[0] != dimsmu[0] || dimsimg[1] != dimsmu[1] || dimsmu[2] != 3 || dimssigma[3] != dimsmu[3])
	  mexErrMsgTxt("mu: wrong dimensions."); 
  if (dimsimg[0] != dimsweights[0] || dimsimg[1] != dimsweights[1] || dimssigma[3] != dimsweights[2])
	  mexErrMsgTxt("weights: wrong dimensions"); 
  
  plhs[0] = mxCreateNumericArray(4, dimssigma, mxDOUBLE_CLASS, mxREAL);
  plhs[1] = mxCreateNumericArray(4, dimsmu, mxDOUBLE_CLASS, mxREAL);
  plhs[2] = mxCreateNumericArray(3, dimsweights, mxDOUBLE_CLASS, mxREAL);
  plhs[3] = mxCreateNumericArray(2, dimsimg, mxDOUBLE_CLASS, mxREAL);    
  if (nlhs == 5)
	  plhs[4] = mxCreateNumericArray(3, dimsimg, mxDOUBLE_CLASS, mxREAL);    
  
  outsigma = mxGetPr(plhs[0]);
  outmu = mxGetPr(plhs[1]);
  outweights = mxGetPr(plhs[2]);  
  foreground = mxGetPr(plhs[3]);  
  background;
  if (nlhs == 5)
	  background = mxGetPr(plhs[4]);  
  
  h = dimssigma[0];
  w = dimssigma[1];
  K = dimssigma[3];
  offset = h * w;
  offset3 = offset * 3;
    
  match = malloc(sizeof(int) * K);
  stddev = malloc(sizeof(double) * K);
  params = malloc(sizeof(gparams) * K);
  
  for (j = 0; j < K; j++)
  {
  	params[j].offset = offset;
    params[j].stddev = stddev;  	
 	}
  
  for (i = 0; i < offset; i++)
  { 	
  	/* Merge equivalent gaussians */
  	/*for (j = 0; j < K-1; j++)
  	{
  	  if (weights[i + offset * j] == 0.)
  	    continue; 	
  		for (k = j+1; k < K; k++)
  		{
	  	  if (weights[i + offset * k] == 0.)
	  	    continue;
  			if (get_density(mu + i + offset3 * k, sigma + i + offset3 * j, mu + i + offset3 * j, offset) > MERGE_DENSITY)
  			{
  			  for (l = 0; l < 3; l++)
  			  {
	  			  mu[i + offset * l + offset3 * j] = weights[i + offset * j] * mu[i + offset * l + offset3 * j] + weights[i + offset * k] * mu[i + offset * l + offset3 * k];
	  			  sigma[i + offset * l + offset3 * k] = pow(weights[i + offset * j], 2) * sigma[i + offset * l + offset3 * j] + pow(weights[i + offset * k], 2) * sigma[i + offset * l + offset3 * k];
	  				mu[i + offset * l + offset3 * k] = -1000;  				
	  				sigma[i + offset * l + offset3 * k] = MIN_SIGMA;    				
  				}
  				weights[i + offset * j] += weights[i + offset * k];
  				weights[i + offset * k] = 0;
  			}  
  		} 
  	}  	*/
    
    /* Get matches */
    nmatch = 0.;
  	for (j = 0; j < K; j++)
  	{
	  	match[j] = (pow(img[i]              - mu[i + offset3 * j], 2)              <= VARTH * sigma[i + offset3 * j] &&
	  	            pow(img[i + offset]     - mu[i + offset + offset3 * j], 2)     <= VARTH * sigma[i + offset + offset3 * j] &&
	  	            pow(img[i + offset * 2] - mu[i + offset * 2 + offset3 * j], 2) <= VARTH * sigma[i + offset * 2 + offset3 * j]);
			stddev[j] = sqrt(sigma[i + offset3 * j] * sigma[i + offset + offset3 * j] * sigma[i + offset * 2 + offset3 * j]);
	  	if (match[j])
		  	nmatch++;
	  }
	  
    /* No matches: update least probable gaussian */
	  if (nmatch == 0.)
	  {
	    int index = 0;	    
	    double weightmin = weights[i];
	    double densitymin = weights[i] * get_normalized_density(img + i, sigma + i, mu + i, offset);
	    if (DEBUG && i == DEBUGPIX)
	    	printf("density 0 : %g\n", densitymin);
	    for (j = 1; j < K; j++)
	    {
	    	double density = weights[i + offset * j] * get_normalized_density(img + i, sigma + i + offset3 * j, mu + i + offset3 * j, offset);
		    if (DEBUG && i == DEBUGPIX)
		    	printf("density %d : %g\n", j, density);	    	
	    	if (density < densitymin || (density == densitymin && weights[i + offset * j] < weightmin))
	    	{
	    		densitymin = density;
	    		weightmin = weights[i + offset * j];
	    		index = j;
	    	}
	    }
	    for (j = 0; j < 3; j++)
	    {
	    	mu[i + offset * j + offset3 * index] = img[i + offset * j];
	    	sigma[i + offset * j + offset3 * index] = NEWVAR;	    	
	    }
	    weights[i + offset * index] = NEWWEIGHT;
	    stddev[index] = sqrt(sigma[i + offset3 * j] * sigma[i + offset + offset3 * j] * sigma[i + offset * 2 + offset3 * j]);
	    match[index] = 1;	      
	  } 	  
  	
  	/* Updates model */
	  sumweight = 0.;  
    for (j = 0; j < K; j++)	  
    {
  		outweights[i + offset * j] = (1. - alpha) * weights[i + offset * j];
    	if (match[j])
    	{
    		const double rho = min(0.99, MULT_VAR * alpha * get_density(img + i, sigma + i + offset3 * j, mu + i + offset3 * j, offset));    		    	
    		for (k = 0; k < 3; k++)
    		{
    			outmu[i + offset * k + offset3 * j] = (1. - rho) * mu[i + offset * k + offset3 * j] + rho * img[i + offset * k];
	    		outsigma[i + offset * k + offset3 * j] = max(MIN_SIGMA, (1. - rho) * sigma[i + offset * k + offset3 * j] + rho * pow(img[i + offset * k] - outmu[i + offset * k + offset3 * j], 2));	    	
    		}
				outweights[i + offset * j] += alpha;
    	}
    	else
    	{
	    	for (k = 0; k < 3; k++)
    		{
		    	outmu[i + offset * k + offset3 * j] = mu[i + offset * k + offset3 * j];
		    	outsigma[i + offset * k + offset3 * j] = sigma[i + offset * k + offset3 * j];
		    }
    	}
    	sumweight += outweights[i + offset * j];
    }
    for (j = 0; j < K; j++)	  
    {
    	params[j].ID = j;
    	params[j].weights = outweights + i;
    	outweights[i + offset * j] /= sumweight;
    }       
       	
    qsort(params, K, sizeof(gparams), (int (*)(const void*, const void*))params_compare);
				
    /* Computes foreground */
		if (nlhs == 5)
	  {
			background[i] = 0.;
			background[i + offset] = 0.;
			background[i + offset * 2] = 0.;
		}
		
		if (DEBUG && i == DEBUGPIX)
		{
   		printf("pix = [%f, %f, %f]\n", img[i], img[i + offset], img[i + 2 * offset]);
			for (j = 0; j < K; j++)
				printf("comp %d:\n - mean = [%f, %f, %f]\n - sigma = [%f, %f %f]\n - stddev = %f\n - weight = %f\n - match = %d\n - w/s = %f\n", j, 
				       outmu[i + 0 + offset3 * j], outmu[i + offset + offset3 * j], outmu[i + 2 * offset + offset3 * j],
				       outsigma[i + 0 + offset3 * j], outsigma[i + offset + offset3 * j], outsigma[i + 2 * offset + offset3 * j], stddev[j],
				       outweights[i + offset * j], match[j], outweights[i + offset * j] / stddev[j]);
			printf("Background comp: ");	       		
		}				

		/*sumback = 0.;
		sumweight = 0.;
		foreground[i] = 0.;		
		for (j = 0; j < K; j++)
			if (sumweight <= T)
			{
				sumweight += outweights[i + offset * params[j].ID];
					
			  if (DEBUG && i == DEBUGPIX)
			    printf("%d ", params[j].ID);

			  sumback += outweights[i + offset * params[j].ID] * get_normalized_density(img + i, outsigma + i + offset3 * params[j].ID, outmu + i + offset3 * params[j].ID, offset);
			  
   			if (nlhs == 5)
					for (k = 0; k < 3; k++)
						background[i + offset * k] += outweights[i + offset * params[j].ID] *  mu[i + offset * k + offset3 * params[j].ID];
			}
			else 
				foreground[i] += outweights[i + offset * params[j].ID] * get_normalized_density(img + i, outsigma + i + offset3 * params[j].ID, outmu + i + offset3 * params[j].ID, offset);
			
	  ratio = foreground[i] / sumback;
		foreground[i] = ratio / (1. + ratio) - 0.5;*/
		
		sumback = 0.;
		sumweight = 0.;
		foreground[i] = 0.;		
		for (j = 0; j < K; j++)
			if (sumweight <= T)
			{
				sumweight += outweights[i + offset * params[j].ID];
					
			  if (DEBUG && i == DEBUGPIX)
			    printf("%d ", params[j].ID);

				if (match[params[j].ID])
				  sumback += outweights[i + offset * params[j].ID];
   			if (nlhs == 5)
					for (k = 0; k < 3; k++)
						background[i + offset * k] += outweights[i + offset * params[j].ID] *  mu[i + offset * k + offset3 * params[j].ID];
			}
			else if (match[params[j].ID])
				foreground[i] += outweights[i + offset * params[j].ID];
			
		foreground[i] = foreground[i] / (foreground[i] + 2. * sumback) - 0.5;
  	if (nlhs == 5)
  	{
			background[i] /= sumweight;
			background[i + offset] /= sumweight;
			background[i + offset * 2] /= sumweight;		
		}
		
		if (DEBUG && i == DEBUGPIX)
		{
		  printf("\nratio = %f\n", ratio);
	  	printf("foreground = %f\n\n", foreground[i]);
	  }
  }    
  
 /* for (i = 0; i < w; i++)
    for (j = 0; j < h; j++)
    {   
    	if (j == 0)
    	{
    		if (PIXC_openN(foreground,i,j,h))
	    		PIX(foreground,i,j,h) = PIXC_openN_sum(foreground,i,j,h) / 3.;
    	}
    	else if (PIX(foreground,i,j,h) * PIX(foreground,i,j-1,h) >= 0 && PIXC_openN(foreground,i,j,h) && PIXC_openS(foreground,i,j-1,h))
    	{
    	  const double mean = (PIXC_openN(foreground,i,j,h) + PIXC_openS(foreground,i,j-1,h)) / 6.;
    		PIX(foreground,i,j,h) = mean;
    		PIX(foreground,i,j-1,h) = mean;
    	} else if (j == h - 1)
    	{
    	  if (PIXC_openS(foreground,i,j,h))
	    		PIX(foreground,i,j,h) = PIXC_openS_sum(foreground,i,j,h) / 3.;
    	}
    	else if (PIX(foreground,i,j,h) == PIX(foreground,i,j+1,h) && PIXC_openS(foreground,i,j,h) && PIXC_openN(foreground,i,j+1,h))
    	{
    	  const double mean = (PIXC_openS(foreground,i,j,h) + PIXC_openN(foreground,i,j+1,h)) / 6.;
    		PIX(foreground,i,j,h) = mean;
    		PIX(foreground,i,j+1,h) = mean;
    	}
    	else if (i == 0)
    	{
    	  if (PIXC_openW(foreground,i,j,h))
	    		PIX(foreground,i,j,h) = PIXC_openW_sum(foreground,i,j,h) / 3.;
    	}
    	else if (PIX(foreground,i,j,h) == PIX(foreground,i-1,j,h) && PIXC_openW(foreground,i,j,h) && PIXC_openE(foreground,i-1,j,h))
    	{
    	  const double mean = (PIXC_openW(foreground,i,j,h) + PIXC_openE(foreground,i-1,j,h)) / 6.;
    		PIX(foreground,i,j,h) = mean;
    		PIX(foreground,i-1,j,h) = mean;
    	}
    	else if (i == w - 1)
    	{
    	  if (PIXC_openE(foreground,i,j,h))
	    		PIX(foreground,i,j,h) = PIXC_openE_sum(foreground,i,j,h) / 3.;
    	}
    	else if (PIX(foreground,i,j,h) == PIX(foreground,i+1,j,h) && PIXC_openE(foreground,i,j,h) && PIXC_openW(foreground,i+1,j,h))
    	{
    	  const double mean = (PIXC_openW(foreground,i,j,h) + PIXC_openE(foreground,i+1,j,h)) / 6.;
    		PIX(foreground,i,j,h) = mean;
    		PIX(foreground,i+1,j,h) = mean;
    	}
    	else if ((PIX(foreground,i,j,h) >= 0 && 
		  		      PIX(foreground,i-1,j,h) <= 0 && PIX(foreground,i+1,j,h) <= 0 &&
                PIX(foreground,i,j-1,h) <= 0 && PIX(foreground,i,j+1,h) <= 0) ||
		  		     (PIX(foreground,i,j,h) <= 0 && 
		  		      PIX(foreground,i-1,j,h) >= 0 && PIX(foreground,i+1,j,h) >= 0 &&
		  		      PIX(foreground,i,j-1,h) >= 0 && PIX(foreground,i,j+1,h) >= 0))  		   
		      		  PIX(foreground,i,j,h) = (PIX(foreground,i-1,j,h) + PIX(foreground,i+1,j,h) + PIX(foreground,i,j-1,h) + PIX(foreground,i,j+1,h)) / 4.;    	
		  
		  PIX(foreground,i,j,h) = max(min(PIX(foreground,i,j,h) + 0.5, 1), 0);    		  
    } */  
  
   for (i = 0; i < w; i++)
    for (j = 0; j < h; j++)
    {
    	/* look for 1-pixel components */
    	if (DEBUG == 2) {printf("a%d_%d", i, j); fflush(stdout);}
    	if (j == 0)
    	{    		    		
    		if (PIXC_openN(foreground,i,j,w,h))
    		{
	    		if (i == 0 || i ==  w - 1)
		    		PIX(foreground,i,j,h) = PIX(foreground,i,j+1,h);
	    		else
		    		PIX(foreground,i,j,h) = PIXC_openN_sum(foreground,i,j,h) / 3.;
		    }		    		
    	}
    	else if (j == h - 1)
    	{
    	  if (PIXC_openS(foreground,i,j,w,h))
    	  {
				 	if (i == 0 || i ==  w - 1)
				 		PIX(foreground,i,j,h) = PIX(foreground,i,j-1,h);
			 		else
		    		PIX(foreground,i,j,h) = PIXC_openS_sum(foreground,i,j,h) / 3.;
		    }
    	}
    	else if (i == 0)
    	{
    	  if (PIXC_openW(foreground,i,j,w,h))
	    		PIX(foreground,i,j,h) = PIXC_openW_sum(foreground,i,j,h) / 3.;
    	}
    	else if (i == w - 1)
    	{
    	  if (PIXC_openE(foreground,i,j,w,h))
	    		PIX(foreground,i,j,h) = PIXC_openE_sum(foreground,i,j,h) / 3.;
    	}   
    	else if ((PIX(foreground,i,j,h) >= 0 && 
		  		      PIX(foreground,i-1,j,h) <= 0 && PIX(foreground,i+1,j,h) <= 0 &&
                PIX(foreground,i,j-1,h) <= 0 && PIX(foreground,i,j+1,h) <= 0) ||
		  		     (PIX(foreground,i,j,h) <= 0 && 
		  		      PIX(foreground,i-1,j,h) >= 0 && PIX(foreground,i+1,j,h) >= 0 &&
		  		      PIX(foreground,i,j-1,h) >= 0 && PIX(foreground,i,j+1,h) >= 0))  		   
		      		  PIX(foreground,i,j,h) = (PIX(foreground,i-1,j,h) + PIX(foreground,i+1,j,h) + PIX(foreground,i,j-1,h) + PIX(foreground,i,j+1,h)) / 4.;  
		      		  
    	if (DEBUG == 2) {printf("b"); fflush(stdout);}
    	/* look for 2-pixels components */    	
    	/* Vertical */
    	if (j > 0)
    	{    	 	
		  	if (PIX(foreground,i,j,h) * PIX(foreground,i,j-1,h) >= 0 && PIXC_openN(foreground,i,j,w,h) && PIXC_openS(foreground,i,j-1,w,h))
		  	{
 		  	  double mean;
		  		if (i > 0 && i < w-1)
		  		{
					  if (j == 1)					  
				  	  mean = PIXC_openN_sum(foreground,i,j,h) / 3.;
				  	else if (j == h-1)
				  	  mean = PIXC_openS_sum(foreground,i,j-1,h) / 3.;				  	
				  	else
				  	  mean = (PIXC_openN_sum(foreground,i,j,h) + PIXC_openS_sum(foreground,i,j-1,h)) / 6.;
					}
					else
					{
						if (j == 1)
							mean = PIX(foreground,i,j+1,h);							
					  else if (j == h-1)
							mean = PIX(foreground,i,j-2,h);
					  else
  					  mean = (PIX(foreground,i,j+1,h) + PIX(foreground,i,j-2,h));							
					}
		  		PIX(foreground,i,j,h) = mean;
		  		PIX(foreground,i,j-1,h) = mean;
		  	}
		 	}
    	if (DEBUG == 2) {printf("c"); fflush(stdout);}		 			 	
    	/* Horizontal */
    	if (i > 0)
    	{    	 	
		  	if (PIX(foreground,i,j,h) * PIX(foreground,i-1,j,h) >= 0 && PIXC_openW(foreground,i,j,w,h) && PIXC_openE(foreground,i-1,j,w,h))
		  	{
 		  	  double mean;
		  		if (j > 0 && j < h-1)
		  		{
					  if (i == 1)					  
				  	  mean = PIXC_openW_sum(foreground,i,j,h) / 3.;
				  	else if (i == w-1)
				  	  mean = PIXC_openE_sum(foreground,i-1,j,h) / 3.;				  	
				  	else
				  	  mean = (PIXC_openW_sum(foreground,i,j,h) + PIXC_openE_sum(foreground,i-1,j,h)) / 6.;
					}
					else
					{
						if (i == 1)
							mean = PIX(foreground,i+1,j,h);							
					  else if (i == w-1)
							mean = PIX(foreground,i-2,j,h);
					  else
  					  mean = (PIX(foreground,i+1,j,h) + PIX(foreground,i-2,j,h));							
					}
		  		PIX(foreground,i,j,h) = mean;
		  		PIX(foreground,i-1,j,h) = mean;
		  	}
		 	}    	    	    	  	
    	if (DEBUG == 2) {printf("d"); fflush(stdout);}
		  PIX(foreground,i,j,h) = max(min(PIX(foreground,i,j,h) + 0.5, 1), 0);    		  
    }   

	if (DEBUG == 2) {printf("e"); fflush(stdout);}  
  
  free(match);
  free(params);
  free(stddev);
}

