function pixelwise_scores = test_dummy_model(params, model, vid)
    % Should return 3D matrix containing the pixelwise score of each label:
    % the i^th 2D slice corresponds to params.annots.labels{i}. The score
    % should be non-negative.

    nlabels = length(model);
       
    % The background image
    back = ssem_load_back(params, vid);
    scale = 800 / size(back, 2);
    back = imresize(back, scale);     
    [h w d] = size(back);

    % Get SIFT features
    [feat featpos] = SIFT_features(back, 8);
    X = featpos(:, 1);
    Y = featpos(:, 2);
    nfeat = size(feat, 1);

    % Compute SVM scores and assign features to the winning label
    scores = zeros(nfeat, nlabels);
    for j = 1 : nlabels
        [~, ~, scores(:, j)] = predict(zeros(nfeat, 1), sparse(double(feat)), model{j});
    end

    % Compute pixel-wise scores for each label by assuming each SIFT is a 16x16 patch
    pixelwise_scores = zeros(h, w, nlabels);
    for k = 1 : nfeat
        [x, y] = meshgrid((X(k) - 7) : (X(k) + 8), (Y(k) - 7) : (Y(k) + 8));
        for j = 1 : nlabels
            pixelwise_scores(y, x, j) = scores(k, j);
        end
    end
    pixelwise_scores = exp(pixelwise_scores);
    
    subplot(2, 2, [1 2]);
    back(back < 0) = 0;
    back(back > 1) = 1;
    imagesc(back);
    axis equal off;
    title('Background image');
    
    subplot(2, 2, 3);
    annot = ssem_load_annots(params, {vid});  
    annot = ssem_unpack_annot(annot{1});        
    imagesc(ssem_scores2img(params, annot, 1));
    axis equal off;    
    title('Ground truth');
    
    subplot(2, 2, 4);
    imagesc(ssem_scores2img(params, pixelwise_scores, 1));
    axis equal off;
    title('Soft segmentation');
    
    pause(0.1);
end
